//------------------------------------------------------------------
//
//------------------------------------------------------------------
#property  indicator_chart_window
#property  indicator_buffers 5
#property  indicator_color1  LimeGreen
#property  indicator_color2  Orange
#property  indicator_color3  Orange
#property  indicator_color4  LimeGreen
#property  indicator_color5  Orange
#property  indicator_width1  2
#property  indicator_width2  2
#property  indicator_width3  2

enum maTypes
{
   ma_sma,    // simple moving average - SMA
   ma_ema,    // exponential moving average - EMA
   ma_smma,   // smoothed moving average - SSMA
   ma_lwma    // linear weighted moving average - LWMA
};
enum enPrice
{
   pr_close,      // Close
   pr_open,       // Open
   pr_high,       // High
   pr_low,        // Low
   pr_median,     // Median
   pr_typical,    // Typical
   pr_weighted    // Weighted
};

extern int     period           = 4;
extern maTypes maType           = ma_ema;
extern enPrice Price            = pr_close;
extern bool    alertsOn         = false;    // Turn alerts on?
extern bool    alertsOnCurrent  = true;     // Alerts on current (still opened) bar?
extern bool    alertsMessage    = true;     // Alerts should show pop-up message?
extern bool    alertsPushNotif  = false;    // Alerts should send push notification?
extern bool    alertsSound      = false;    // Alerts should play a sound?
extern bool    alertsEmail      = false;    // Alerts should send email?
extern bool    showLines        = true;
extern bool    showArrows       = true;

double BufferUp[],BufferDn[], ma[], maofma[], maofmada[], maofmadb[], slope[];

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int init()
{
   int lstyle = DRAW_LINE;  if (!showLines)  lstyle = DRAW_NONE;
   int astyle = DRAW_ARROW; if (!showArrows) astyle = DRAW_NONE;
   IndicatorBuffers(7);
      SetIndexBuffer(0,maofma);   SetIndexStyle(0,lstyle);
      SetIndexBuffer(1,maofmada); SetIndexStyle(1,lstyle);
      SetIndexBuffer(2,maofmadb); SetIndexStyle(2,lstyle);
      SetIndexBuffer(3,BufferUp); SetIndexStyle(3,astyle); SetIndexArrow(3,233);
      SetIndexBuffer(4,BufferDn); SetIndexStyle(4,astyle); SetIndexArrow(4,234);   
      SetIndexBuffer(5,ma);
      SetIndexBuffer(6,slope);
   return(0);
}
int deinit() { return(0);}  

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int start()
{
   int i,counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
           int limit=MathMin(Bars-counted_bars,Bars-1);

   if (slope[limit]==-1) CleanPoint(limit,maofmada,maofmadb);
   for(i=limit; i>=0; i--) ma[i] = iMA(NULL,0,period,0,(int)maType,(int)Price,i);
   for(i=limit; i>=0; i--)
   {
      maofma[i]   = iMAOnArray(ma,0,period,0,(int)maType,i); 
      maofmada[i] = EMPTY_VALUE;
      maofmadb[i] = EMPTY_VALUE;
      BufferDn[i] = EMPTY_VALUE;
      BufferUp[i] = EMPTY_VALUE;
      slope[i]    = slope[i+1];
         if (maofma[i]>maofma[i+1]) slope[i] =  1;
         if (maofma[i]<maofma[i+1]) slope[i] = -1;
         if (slope[i] == -1) PlotPoint(i,maofmada,maofmadb,maofma);
         if (slope[i] != slope[i+1])
            if (slope[i] == 1)
                  BufferUp[i] = Low[i]  - iATR(NULL,0,20,i);
            else  BufferDn[i] = High[i] + iATR(NULL,0,20,i);
   } 
   manageAlerts();
   return(0);
}


//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

//
//
//
//
//

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (i>=Bars-2) return;
   if (first[i+1] == EMPTY_VALUE)
         {
            if (first[i+2] == EMPTY_VALUE) 
                  { first[i]  = from[i]; first[i+1]  = from[i+1]; second[i] = EMPTY_VALUE; }
            else  { second[i] = from[i]; second[i+1] = from[i+1]; first[i]  = EMPTY_VALUE; }
         }
   else  { first[i] = from[i]; second[i] = EMPTY_VALUE; }
}

//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      int whichBar = 1; if (alertsOnCurrent) whichBar = 0;
      if (slope[whichBar] != slope[whichBar+1])
      {
         if (slope[whichBar]== 1) doAlert(whichBar,"slope changed to up");
         if (slope[whichBar]==-1) doAlert(whichBar,"slope changed to down");
      }
   }
}

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message = Symbol()+" at "+TimeToStr(TimeLocal(),TIME_SECONDS)+" ma of ma "+doWhat;
          if (alertsMessage)   Alert(message);
          if (alertsEmail)     SendMail(StringConcatenate(Symbol()," ma of ma "),message);
          if (alertsPushNotif) SendNotification(StringConcatenate(Symbol()," ma of ma "+message));
          if (alertsSound)     PlaySound("alert2.wav");
   }
}